from __future__ import absolute_import
from .zjwbox import *
from .auto_req import *
from .comm_req import *
import os
from pyppeteer import chromium_downloader


name = "zjwbox"
version = "0.1.8"


pyppeteer_version_win32 = chromium_downloader.downloadURLs.get("win32")
pyppeteer_version_win64 = chromium_downloader.downloadURLs.get("win64")





import requests
import parsel, re
import time, random
import asyncio, aiohttp
from tqdm import tqdm
from itertools import product
import eventlet
from faker import Faker
from datetime import datetime


fake = Faker()
fake_ua = fake.user_agent()

class Comm_req(object):

    def __init__(self, urls, headers={"UserAgent": fake_ua}, cookie=None, data=None, 
                        encode="utf-8", mode="get", time_inter=False):
        self.urls = urls
        self.headers = headers
        self.cookie = cookie
        self.data = data
        self.mode = mode
        self.encode = encode
        self.time_inter = time_inter

    async def fask(self, url, mode="get") -> str:
        async with aiohttp.ClientSession() as session:
            if self.mode == "post":
                async with session.post(url, headers=self.headers, timeout=3) as res:
                    return await res.text()
            else:
                async with session.get(url, headers=self.headers, timeout=3) as res:
                    return await res.text()

    def deal(self):
        if isinstance(self.urls, str):
            session = requests.Session()
            self.time_inter = 0
            if self.time_inter:
                self.time_inter = random.choice([0.5, 1, 0.9, 1.5])
                res = session.get(self.urls, headers=self.headers, timeout=5)
                text = res.content.decode(self.encode)
                if self.mode == "post":
                    res = session.post(self.urls, self.headers)
                    text = res.content.decode(self.encode)
                time.sleep(self.time_inter)
                yield text
            else:
                res = session.get(self.urls, headers=self.headers, timeout=5)
                text = res.content.decode(self.encode)
                if self.mode == "post":
                    res = session.post(self.urls, self.headers)
                    text = res.content.decode(self.encode)
                yield text

        else:
            for url in self.urls:
                loop = asyncio.get_event_loop()
                text = loop.run_until_complete( self.fask(url, self.mode) )
                now_time = datetime.now()
                print(f"爬取{url}成功！时间：{now_time}")
                yield text


    def get_page(self):
        result = []
        for page in self.deal():
            result.append(page)
        if result == []:
            return None
        eventlet.monkey_patch()
        now_time = datetime.now()
        if isinstance(self.urls, str):
            print(f"目标1个，成功爬取{len(result)}, 时间：{now_time}")
        else:
            print(f"目标{len(self.urls)}个，成功爬取{len(result)}，时间：{now_time}")
  
        return result
        
           

if __name__ == "__main__":
    urls = ["https://www.baidu.com", "https://www.douban.com", "https://www.zhihu.com"]
    # urls = "https://www.baidu.com"
    c = Comm_req(urls=urls)
    text = c.get_page()
    print(text)

    





from dataclasses import dataclass
from collections import Counter

@dataclass
class Infer(object):
    data_type: str
    
    def __str__(self):
        info =         """
        welcome you!
        This is a class that improve infer!
        You must give it a arg to str type, "list", "tuple" or "set".
        """
        return info
    
    def __getitem__(self, key):
        if len(str(key)) < 2:
            print("语法错误：输入格式数不能少于2！")
            return
        if isinstance(key, tuple) and len(key) in (2, 3, 4) and (key[2] is Ellipsis or key[1] is Ellipsis or key[3] is Ellipsis):
            if key[1] is Ellipsis:
                basic_infer = [i for i in range(key[0], key[-1] + 1)]
            elif key[3] is Ellipsis:
                basic_infer = [i for i in range(key[0], key[-1] + 1, key[1] - key[0])]
            else:
                basic_infer = [i for i in range(key[0], key[-1] + 1, key[1] - key[0])]
            
            if self.data_type == "list":
                return basic_infer
            if self.data_type == "tuple":
                return tuple(basic_infer)
            if self.data_type == "set":
                return set(basic_infer)
        timers = Counter(self.data_type).most_common()
        for timer in timers:
            if 2 in timer:
                print(f"请检查参数: {self.data_type}\n参考参数：'list', 'tuple', 'set'！")
                return
        if len(self.data_type) < 3 or len(self.data_type) > 5:
            print(f"请检查参数: {self.data_type}\n参考参数：'list', 'tuple', 'set'！")
        else:
            print(None)
                

                
if __name__ == "__main__":
    p = Infer("set")
    print( p[1, 5,  ...,100] )


from dataclasses import dataclass
import pymongo, pymysql
from tqdm import tqdm, trange
import csv, openpyxl
from collections.abc import Iterable
from threading import Lock



"""
welcome you !
This is a class about store data !
You might use it store many different type for your data
For example, txt, Csv, Excel, a kind of Databases !
You should give it some args, text of data, path of file and name of file,
It is necessary to some args, your data in text, path of file, filename.
"""


@dataclass
class Store(object):
    text: list or dict
    path: str
    filename:str

    _instance_lock = Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(Store, "_instance"):
            with Store._instance_lock:
                if not hasattr(Store, "_instance"):
                    Store._instance = object.__new__(cls)  
        return Store._instance


    def __str__(self) -> str:
        info = """
                This is a class about store data !
                You able use it store many different type for your data !
                For example, txt, Csv, Excel, a kind of Databases !
                """
        return info


    """
    Save to your TXT !
    you may give some choice args, for example,  encode of char, 
    even can change is_mark that decide read, write or add to !
    """
    def to_txt(self, encode: str="utf-8", is_mark=True ) -> all:
        if isinstance(self.text, list):
            words, count = 0, 0
            mode = "a+"
            if is_mark:
                mode = "w+"
            with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode) as f:
                for i in tqdm(self.text):
                    f.writelines(i)
                    count += 1
                    words += len(i)
            print(f"保存完成！location of file：{self.path}/{self.filename}，共{count}行，{words}字！")
        else:
            print("TypeErorr: text must be list !")


    """
    Save to your CSV !
    you can give some choice args for this function, for example, mode to read and write, 
    if your datas is dict or set of dict, you may give a arg of headers(title in csv),
    you even can change is_single that deal most datas !
    """
    def to_csv(self, mode="w+", encode: str="utf-8", is_single=True, headers=None)-> None :
        if ( isinstance(self.text, list) and not isinstance(self.text[0], dict) ) or isinstance(self.text[0], list):
            if mode == "r":
                with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode) as f:
                    reader = csv.reader(f)
                    for row in reader:
                        print(row)
            with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode, newline="") as f:
                writer = csv.writer(f)
                if isinstance(self.text[0], list):
                    if is_single:
                        for t in tqdm(self.text):
                            writer.writerow(t)
                            tqdm(self.text).set_description("正在写入中！")
                    else:
                        for t in tqdm(self.text):
                            writer.writerows(t)
                            tqdm(self.text).set_description("正在写入中！")
                    print("写入完成！")

        elif isinstance(self.text, dict) or isinstance(self.text[0], dict):
            if mode == "r":
                with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode) as f:
                    reader = csv.reader(f)
                    for row in reader:
                        print(row)
            else:
                with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode, newline="") as f:
                    Dictwriter = csv.DictWriter(f, headers)
                    Dictwriter.writeheader()
                    if  isinstance(self.text[0], dict):
                        if is_single:
                            for d in tqdm(self.text):
                                Dictwriter.writerow(d)
                                tqdm(self.text).set_description("正在写入中！")
                        else:
                            for d in tqdm(self.text):
                                Dictwriter.writerows(d)
                                tqdm(self.text).set_description("正在写入中！")
                        print("写入完成！")
                    else:
                        if is_single:
                            Dictwriter.writerow(self.text)
                            print("写入完成！")
                        else:
                            for row in tqdm(self.text):
                                Dictwriter.writerows(row)
                                tqdm(self.text).set_description("正在写入中！")
                            print("写入完成！")

        else:
            print("TypeErorr: text must be list or dict !")


    """
    Save to your Excel !
    This function have a necessary arg to sheet_name .
    of couser, you also give some choice args, for example, mode to read and write, 
    excel that row, column and value,
    you even can change is_single that deal most datas !
    """
    def ente_excel(func) -> all:
        def warpper(self, *args, **kwargs):
            func(self, *args, **kwargs)
        return warpper

    @ente_excel
    def to_excel(self, sheet_name: str, mode="w", row=None, column=None, value=None, is_single=False) -> None:
        wb = openpyxl.Workbook()
        sh = wb.create_sheet(sheet_name)
        if mode == "w":
            if not isinstance(self.text[0], Iterable):
                row, column, value = self.text
                sh.cell(row=row, column=column, value=value)
                wb.save( f"{self.path}/{self.filename}" )
                wb.close()
                print("保存成功！")
                return
            else:
                for t in tqdm(self.text):
                    row, column, value = t
                    sh.cell(row=row, column=column, value=value)
                    tqdm(self.text).set_description("正在保存中！")
                wb.save( f"{self.path}/{self.filename}" )
                wb.close()
                print("保存成功！")
                return

        wb = openpyxl.load_workbook(f"{self.path}/{self.filename}")
        sh = wb[sheet_name]
        #按行读取数据转化为列表
        rows_data = list(sh.rows)
        # print(rows_data)
        # 获取表单的表头信息
        titles = []
        for title in rows_data[0]:
            titles.append(title.value)
        cases = []
        for case in rows_data[1:]:
            data = []
            for cell in case: 
                data.append(cell.value)
                if isinstance(cell.value,str):
                    data.append(cell.value)
                else:
                    data.append(cell.value)
                case_data = dict(  list (zip(titles,data) )  )
                cases.append(case_data)
        return cases


    def to_db(self):
        ...


if __name__ == "__main__":
    text = [{"1":1}, {"2":2}]
    headers = ["1", "2"]
    test_excel =[ [1 , 1 , "3"], [1 , 2 , "3"], [1 , 3 , "3"] ]  
    s = Store(path="E:", filename="store_test.xlsx", text=test_excel)
    s.to_excel(sheet_name="Sheet1")


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

    
setuptools.setup(
    name="zjwbox",
    version = "0.1.8",
    author = "zjw",
    author_email="2415528031@qq.com",
    description="a package for extend",
    long_description=long_description,
    # long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    install_requires=["wget==3.2", "lxml==4.6.1", "eventlet==0.29.1", "dataclasses", "pyppeteer", "aiohttp==3.7.1", "nest_asyncio==1.4.1", "parsel", "requests", "faker", "openpyxl"]

)



import requests
import parsel, re
from lxml import etree
import time, random
from pyppeteer import launch, launcher
import asyncio
import nest_asyncio
import warnings, re
from tqdm import tqdm, trange
from faker import Faker



warnings.filterwarnings("ignore")
nest_asyncio.apply()


source_pages = []
window_pages = []
launcher.DEFAULT_ARGS.remove("--enable-automation")

async def source_page(url: str) -> None:
    browser = await launch({"headless":False,"dumpio":True,"autoClose":False,"args":["--no-sandbox","--window-size=1366.850"]})
    page = await browser.newPage()
    await page.goto(url)
    text = await page.content()
    source_pages.append(text)
    await browser.close()


async def window_page(url: str) -> None:
    temp_str = ""
    rand_UA = []
    fake = Faker()
    for _ in range(10):
        rand_UA.append( fake.user_agent() )
    pypp_UA = random.choice(rand_UA)
    browser = await launch(
        {
             "headless":False,
             "dumpio":True,
             "autoClose":False,
             "userDataDir":r"D:\\temporary",
              "args":[
                  "--no-sandbox",
                  "--window-size=1366.850",
                  f"--user-agent={pypp_UA}"
              ]
        }
    )
    page = await browser.newPage()
    await page.goto(url)

    try:
        cont_click = await page.querySelector(" div.fold-page-text > span")
        await cont_click.click()
    except:
        pass

    try:
        p_labels = await page.querySelectorAll(selector=" div > div > p")
        for p in p_labels:
            p_text = await (await p.getProperty("textContent")).jsonValue()
            if p_text == " ":
                pass
            else:
                temp_str += p_text.strip()
        window_pages.append(temp_str)
        await browser.close()
    except Exception as e:
        print(f"浏览器发生错误！\n{e}")


def get_window_page(urls):
    for url in urls:
        asyncio.get_event_loop().run_until_complete(window_page(url))
    return window_pages


def get_source_page(urls: list) -> list:
    for url in urls:
        asyncio.get_event_loop().run_until_complete(source_page(url))
    return source_pages


def extra(text, type_extra="font", super_mode=False) -> all:
    if type_extra == "own":
        selector = parsel.Selector(text)
        xpath = etree.HTML(text)
        return selector, xpath

    if super_mode:
        pass

    if type_extra == "font":
        font_text = []
        string = text
        pattern = r"<p>(.*?)</p>"
        font_texts = re.findall(string=string, pattern=pattern)
        for f in font_texts:
            if "&nbsp" or "<br>" or "<br/>" or "<" in f:
                deal_f = re.sub(string=f, pattern=r"[&nbsp, (<\[a-z\]>.*?<\[a-z\]>)]", repl="")
                font_text.append(deal_f.strip())
            else:
                font_text.append(f)

        return font_text




if __name__ == "__main__":
    # urls = ["https://blog.csdn.net/weixin_41846313/article/details/84718739"]
    urls = ["https://wenku.baidu.com/view/7bb441e6a88271fe910ef12d2af90242a995ab1c.html?fr=aladdin664466&ind=1"]
    # selector, xpath = extra(text=text[0], type_extra="own")
    # print(selector, xpath)
    text = get_window_page(urls)
    print(text)